@@ -1,8 +1,21 @@
 """
 PyDMD init
 """
-__all__ = ['dmdbase', 'dmd', 'fbdmd', 'mrdmd', 'cdmd', 'hodmd', 'dmdc',
-           'optdmd', 'hankeldmd']
+__all__ = [
+    "dmdbase",
+    "dmd",
+    "fbdmd",
+    "mrdmd",
+    "cdmd",
+    "hodmd",
+    "dmdc",
+    "optdmd",
+    "hankeldmd",
+    "rdmd",
+    "havok",
+    "bopdmd",
+    "pidmd",
+]
 
 
 from .meta import *
@@ -19,3 +32,7 @@ from .spdmd import SpDMD
 from .paramdmd import ParametricDMD
 from .dmd_modes_tuner import ModesTuner
 from .subspacedmd import SubspaceDMD
+from .rdmd import RDMD
+from .havok import HAVOK
+from .bopdmd import BOPDMD
+from .pidmd import PiDMD
