@@ -5,17 +5,15 @@ Reference:
 - Kutz, J. Nathan, Xing Fu, and Steven L. Brunton. Multiresolution Dynamic Mode
 Decomposition. SIAM Journal on Applied Dynamical Systems 15.2 (2016): 713-735.
 """
-from __future__ import division
-from functools import partial
-from builtins import range
-from past.utils import old_div
-import numpy as np
-import matplotlib.pyplot as plt
 from copy import deepcopy
+from functools import partial
+
+import numpy as np
 from scipy.linalg import block_diag
 
+from .dmd_modes_tuner import select_modes
 from .dmdbase import DMDBase
-from .dmd_modes_tuner import ModesSelectors, select_modes
+from .snapshots import Snapshots
 
 
 class BinaryTree:
@@ -39,14 +37,14 @@ class BinaryTree:
                 )
             )
 
-        if bin_ >= 2 ** level_:
+        if bin_ >= 2**level_:
             raise ValueError("Invalid node")
 
-        return self.tree[2 ** level_ + bin_ - 1]
+        return self.tree[2**level_ + bin_ - 1]
 
     def __setitem__(self, val, item):
         level_, bin_ = val
-        self.tree[2 ** level_ + bin_ - 1] = item
+        self.tree[2**level_ + bin_ - 1] = item
 
     def __iter__(self):
         return self.tree.__iter__()
@@ -56,7 +54,7 @@ class BinaryTree:
         return range(self.depth + 1)
 
     def index_leaves(self, level):
-        return range(0, 2 ** level)
+        return range(0, 2**level)
 
 
 class MrDMD(DMDBase):
@@ -257,14 +255,11 @@ Expected one item per level, got {} out of {} levels.""".format(
         indexes = []
         for level in self.dmd_tree.levels:
             for leaf in self.dmd_tree.index_leaves(level):
-
                 local_times = self.partial_time_interval(level, leaf)
                 if (
                     local_times["t0"] <= t0 < local_times["tend"]
                     or local_times["t0"] < tend <= local_times["tend"]
-                    or (
-                        t0 <= local_times["t0"] and tend >= local_times["tend"]
-                    )
+                    or (t0 <= local_times["t0"] and tend >= local_times["tend"])
                 ):
                     indexes.append((level, leaf))
 
@@ -295,9 +290,7 @@ Expected one item per level, got {} out of {} levels.""".format(
         :rtype: numpy.ndarray
         """
         indexes = self.time_window_bins(t0, tend)
-        return np.concatenate(
-            [self.dmd_tree[idx].frequency for idx in indexes]
-        )
+        return np.concatenate([self.dmd_tree[idx].frequency for idx in indexes])
 
     def time_window_growth_rate(self, t0, tend):
         """
@@ -429,11 +422,11 @@ Expected one item per level, got {} out of {} levels.""".format(
                 )
             )
 
-        if leaf >= 2 ** level:
+        if leaf >= 2**level:
             raise ValueError("Invalid node")
 
         full_period = self.original_time["tend"] - self.original_time["t0"]
-        period = full_period / 2 ** level
+        period = full_period / 2**level
         t0 = self.original_time["t0"] + period * leaf
         tend = t0 + period
         return {"t0": t0, "tend": tend, "delta": period}
@@ -460,11 +453,13 @@ Expected one item per level, got {} out of {} levels.""".format(
         :param X: the input snapshots.
         :type X: numpy.ndarray or iterable
         """
-        self._snapshots, self._snapshots_shape = self._col_major_2darray(X)
+        self._reset()
+
+        self._snapshots_holder = Snapshots(X)
 
         # Redefine max level if it is too big.
         lvl_threshold = (
-            int(np.log(self._snapshots.shape[1] / 4.0) / np.log(2.0)) + 1
+            int(np.log(self.snapshots.shape[1] / 4.0) / np.log(2.0)) + 1
         )
         if self.max_level > lvl_threshold:
             self.max_level = lvl_threshold
@@ -477,16 +472,16 @@ Expected one item per level, got {} out of {} levels.""".format(
         def slow_modes(dmd, rho):
             return np.abs(np.log(dmd.eigs)) < rho * 2 * np.pi
 
-        X = self._snapshots.copy()
+        X = self.snapshots.copy()
         for level in self.dmd_tree.levels:
-            n_leaf = 2 ** level
+            n_leaf = 2**level
             Xs = np.array_split(X, n_leaf, axis=1)
 
             for leaf, x in enumerate(Xs):
                 current_dmd = self.dmd_tree[level, leaf]
                 current_dmd.fit(x)
 
-                rho = old_div(float(self.max_cycles), x.shape[1])
+                rho = self.max_cycles / x.shape[1]
                 slow_modes_selector = partial(slow_modes, rho=rho)
 
                 select_modes(current_dmd, slow_modes_selector)
@@ -500,116 +495,7 @@ Expected one item per level, got {} out of {} levels.""".format(
             X -= newX
 
         self._set_initial_time_dictionary(
-            dict(t0=0, tend=self._snapshots.shape[1], dt=1)
+            dict(t0=0, tend=self.snapshots.shape[1], dt=1)
         )
 
         return self
-
-    def plot_eigs(
-        self,
-        show_axes=True,
-        show_unit_circle=True,
-        figsize=(8, 8),
-        title="",
-        level=None,
-        node=None,
-    ):
-        """
-        Plot the eigenvalues.
-
-        :param bool show_axes: if True, the axes will be showed in the plot.
-                Default is True.
-        :param bool show_unit_circle: if True, the circle with unitary radius
-                and center in the origin will be showed. Default is True.
-        :param tuple(int,int) figsize: tuple in inches of the figure.
-        :param str title: title of the plot.
-        :param int level: plot only the eigenvalues of specific level.
-        :param int node: plot only the eigenvalues of specific node.
-        """
-        if self.eigs is None:
-            raise ValueError(
-                "The eigenvalues have not been computed."
-                "You have to perform the fit method."
-            )
-
-        if level:
-            peigs = self.partial_eigs(level=level, node=node)
-        else:
-            peigs = self.eigs
-
-        plt.figure(figsize=figsize)
-        plt.title(title)
-        plt.gcf()
-        ax = plt.gca()
-
-        if not level:
-            cmap = plt.get_cmap("viridis")
-            colors = [
-                cmap(i) for i in np.linspace(0, 1, len(self.dmd_tree.levels))
-            ]
-
-            points = []
-            for level in self.dmd_tree.levels:
-                eigs = self.partial_eigs(level)
-
-                points.append(
-                    ax.plot(eigs.real, eigs.imag, ".", color=colors[level])[0]
-                )
-        else:
-            points = []
-            points.append(
-                ax.plot(peigs.real, peigs.imag, "bo", label="Eigenvalues")[0]
-            )
-
-        # set limits for axis
-        limit = np.max(np.ceil(np.absolute(peigs)))
-        ax.set_xlim((-limit, limit))
-        ax.set_ylim((-limit, limit))
-
-        plt.ylabel("Imaginary part")
-        plt.xlabel("Real part")
-
-        if show_unit_circle:
-            unit_circle = plt.Circle(
-                (0.0, 0.0), 1.0, color="green", fill=False, linestyle="--"
-            )
-            ax.add_artist(unit_circle)
-
-        # Dashed grid
-        gridlines = ax.get_xgridlines() + ax.get_ygridlines()
-        for line in gridlines:
-            line.set_linestyle("-.")
-        ax.grid(True)
-
-        ax.set_aspect("equal")
-
-        # x and y axes
-        if show_axes:
-            ax.annotate(
-                "",
-                xy=(np.max([limit * 0.8, 1.0]), 0.0),
-                xytext=(np.min([-limit * 0.8, -1.0]), 0.0),
-                arrowprops=dict(arrowstyle="->"),
-            )
-            ax.annotate(
-                "",
-                xy=(0.0, np.max([limit * 0.8, 1.0])),
-                xytext=(0.0, np.min([-limit * 0.8, -1.0])),
-                arrowprops=dict(arrowstyle="->"),
-            )
-
-        # legend
-        if level:
-            labels = ["Eigenvalues - level {}".format(level)]
-        else:
-            labels = [
-                "Eigenvalues - level {}".format(i)
-                for i in range(self.max_level)
-            ]
-
-        if show_unit_circle:
-            points += [unit_circle]
-            labels += ["Unit circle"]
-
-        ax.add_artist(plt.legend(points, labels, loc="best"))
-        plt.show()
